
/* Generated data (by glib-mkenums) */

#ifndef __EXO_HELPER_ENUM_TYPES_H__
#define __EXO_HELPER_ENUM_TYPES_H__
#include <exo/exo.h>
G_BEGIN_DECLS
/* enumerations from "exo-helper.h" */
GType exo_helper_category_get_type (void) G_GNUC_CONST;
#define EXO_TYPE_HELPER_CATEGORY (exo_helper_category_get_type())
G_END_DECLS

#endif /* !__EXO_HELPER_ENUM_TYPES_H__ */

/* Generated data ends here */

